
// Na nasledujicich radcich jsou nacteny knihovny
#include <Console.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <Wire.h>

#define OPEN_DELAY        30 // Nastaveni prodlevy pro spusteni alarmu pri otevrene lednici
#define LIGHT_THRESHOLD  420 // Kalibrace senzoru svetla

// Na nasledujicich radcich jsou globalni promenne
uint8_t open_delay = 0, fridge_open = 0; // otevirani lednicky
uint8_t temperature_unit; // jednotky
int8_t temperature, temperature_max, temperature_min; // teploty, max, min
int8_t temperature_max2, temperature_min2; // mezni hodnoty
uint8_t switch_threshold = 0; // Switches
uint8_t year, month, day, hour, minute, sec; // Time
uint8_t serialbuffer[50], bufferindex = 0; // Serial
uint16_t timeout = 0; // Serial
uint8_t alarm = 0; // prekroceni meznich hodnot
uint16_t ledtimeout = 0, displaytimeout = 0;  // prekroceni meznich hodnot

// Look-up tables for TM1636
static int8_t lookup[] = {0x3f,0x06,0x5b,0x4f,
                          0x66,0x6d,0x7d,0x07,
                          0x7f,0x6f,0x77,0x7c,
                          0x39,0x5e,0x79,0x71,
                          0x40,0x00};
static int8_t errorlookup[] = {0x79,0x77,0x77,0x3f,0x77,0x00,0x79,0x77,0x77,0x3f,0x77};

// Prvni vec, ktera se na arduinu spusti
void setup() {

  // Nastaveni serioveho portu - Uzivatel musi pristupovat k portu s touto rychlosti
  Serial.begin(9600);
  while(Serial.available()){Serial.read();} // vycisti buffer
  
  // Nastavi i2c linku - RTC
  Wire.begin();
  
  // Nasledujici radky nastavi IO porty arduina
  DDRD |= 252; // LED, BUZZER, TM1636
  DDRB |= 1;   // Tlacitka, TM1636
  PORTB = 14;  // Pullup pro tlacitka

  // Nasledujici radky nastavi timer1 na frekvenci 1HZ, ale nespusti ho
  OCR1A = 15624; // 1Hz
  TCCR1A = 0;
  TCCR1B = (1<<WGM12); // ctc, prescaller 1024
  TIMSK1 = (1<<OCIE1A);

  // Nasledujici radky nastavi timer2 na frekvenci 200hz, ale nespusti ho - pro LED (frekvenec se deli deseti a pak dvema)
  OCR2A = 77;
  TCCR2A |= (1<<WGM21);
  TCCR2B = 0;
  TIMSK2 |= (1<<OCIE2A);

  // Precte se EEPROM stav jednotek - C/F, a maximalni minimalni teploty a mezni hodnoty
  temperature_unit = eeprom_read_byte((uint8_t*)0);
  temperature_max = (int8_t)eeprom_read_byte((uint8_t*)1);
  temperature_min = (int8_t)eeprom_read_byte((uint8_t*)2);
  temperature_max2 = (int8_t)eeprom_read_byte((uint8_t*)21);
  temperature_min2 = (int8_t)eeprom_read_byte((uint8_t*)22);
  
  // Povolit preruseni
  sei();

}

// Spocti 30 sekund a udelej frekvenci 10hz pro buzzer, frekvence se deli dvema
ISR(TIMER1_COMPA_vect){
  if(open_delay < 200)
    open_delay++;
  if(fridge_open && open_delay >= OPEN_DELAY){
    PORTD ^= 1<<6;
  }
}

// Udelej 20Hz pro ledky a 2Hz pro display, frekvence se deli dvema
ISR(TIMER2_COMPA_vect){
  TCNT2 = 0;
  ledtimeout++;
  displaytimeout++;
  if(ledtimeout > 9){
    ledtimeout = 0;
    PORTD ^= 60;
  }
  if(displaytimeout == 99){
    display(0, lookup[17]);
    display(1, lookup[17]);
    display(2, lookup[17]);
    display(3, lookup[17]);
  }
  if(displaytimeout >= 199){
    displaytimeout = 0;
    displayTemperature(temperature);
  }
}

// Nekonecna smycka
void loop() {

  //Komunikace se seriovou linkou
  while(Serial.available()){
    uint8_t b = Serial.read();
    if(b != 10 && b != 13){ // Ignoruj konec radku
      if(b >= 97)
        b -= 32; // preved na velka pismena
      serialbuffer[bufferindex] = b; // uloz do bufferu
      bufferindex++;
    }
  }
  if(timeout > 100){ // timeout pro seriovou komunikaci
    if(bufferindex > 0){
      if(bufferindex == 4 && serialbuffer[0] == 'M' && serialbuffer[1] == 'A' && serialbuffer[2] == 'X' && serialbuffer[3] == 'C'){
        GetTimeEEPROM(3);
        printTime();
        Serial.print(temperature_max); Serial.println("C");
      } else if(bufferindex == 4 && serialbuffer[0] == 'M' && serialbuffer[1] == 'A' && serialbuffer[2] == 'X' && serialbuffer[3] == 'F'){
        GetTimeEEPROM(3);
        printTime();
        Serial.print(convertToF(temperature_max)); Serial.println("F");
      } else if(bufferindex == 4 && serialbuffer[0] == 'M' && serialbuffer[1] == 'I' && serialbuffer[2] == 'N' && serialbuffer[3] == 'C'){
        GetTimeEEPROM(10);
        printTime();
        Serial.print(temperature_min); Serial.println("C");
      } else if(bufferindex == 4 && serialbuffer[0] == 'M' && serialbuffer[1] == 'I' && serialbuffer[2] == 'N' && serialbuffer[3] == 'F'){
        GetTimeEEPROM(10);
        printTime();
        Serial.print(convertToF(temperature_min)); Serial.println("F");
      } else if(bufferindex == 4 && serialbuffer[0] == 'A' && serialbuffer[1] == 'L' && serialbuffer[2] == 'L' && serialbuffer[3] == 'C'){
        GetTimeEEPROM(10);
        printTime();
        Serial.print(temperature_min); Serial.println("C");
        GetTimeEEPROM(3);
        printTime();
        Serial.print(temperature_max); Serial.println("C");
      } else if(bufferindex == 4 && serialbuffer[0] == 'A' && serialbuffer[1] == 'L' && serialbuffer[2] == 'L' && serialbuffer[3] == 'F'){
        GetTimeEEPROM(10);
        printTime();
        Serial.print(convertToF(temperature_min)); Serial.println("F");
        GetTimeEEPROM(3);
        printTime();
        Serial.print(convertToF(temperature_max)); Serial.println("F");
      } else if(bufferindex == 6 && serialbuffer[0] == 'D' && serialbuffer[1] == 'E' && serialbuffer[2] == 'L' && serialbuffer[3] == 'E' && serialbuffer[4] == 'T' && serialbuffer[5] == 'E'){
        eeprom_write_byte((uint8_t*)1, 0);
        eeprom_write_byte((uint8_t*)2, 127);
        temperature_max = 0;  temperature_min = 127;
        sec = 0; minute = 0; hour = 0; day = 0; month = 0; year = 0;
        SaveTime(3); SaveTime(10);
        Serial.println("OK");
      } else if(bufferindex == 15 && serialbuffer[0] == 'D' && serialbuffer[1] == 'E' && serialbuffer[2] == 'L' && serialbuffer[3] == 'E' && serialbuffer[4] == 'T' && serialbuffer[5] == 'E' && 
                serialbuffer[6] == 'L' && serialbuffer[7] == 'I' && serialbuffer[8] == 'M' && serialbuffer[9] == 'I' && serialbuffer[10] == 'T' && serialbuffer[11] == 'M' &&
                serialbuffer[12] == 'A' && serialbuffer[13] == 'X' && serialbuffer[14] == 'C'){
        eeprom_write_byte((uint8_t*)21, 127);
        temperature_max2 = 127;
        Serial.println("OK");
      } else if(bufferindex == 15 && serialbuffer[0] == 'D' && serialbuffer[1] == 'E' && serialbuffer[2] == 'L' && serialbuffer[3] == 'E' && serialbuffer[4] == 'T' && serialbuffer[5] == 'E' && 
                serialbuffer[6] == 'L' && serialbuffer[7] == 'I' && serialbuffer[8] == 'M' && serialbuffer[9] == 'I' && serialbuffer[10] == 'T' && serialbuffer[11] == 'M' &&
                serialbuffer[12] == 'I' && serialbuffer[13] == 'N' && serialbuffer[14] == 'C'){
        eeprom_write_byte((uint8_t*)22, 0);
        temperature_min2 = 0;
        Serial.println("OK");
      } else if(bufferindex == 14 && serialbuffer[0] == 'S' && serialbuffer[1] == 'E' && serialbuffer[2] == 'T' && serialbuffer[3] == 'L' && serialbuffer[4] == 'I' && serialbuffer[5] == 'M' && 
                serialbuffer[6] == 'I' && serialbuffer[7] == 'T' && serialbuffer[8] == 'M' && serialbuffer[9] == 'A' && serialbuffer[10] == 'X' && serialbuffer[11] == 'C' &&
                serialbuffer[12] == ' ' && serialbuffer[13] >= '0' && serialbuffer[13] <= '9'){
        temperature_max2 = serialbuffer[13]-48;
        eeprom_write_byte((uint8_t*)21, temperature_max2);
        Serial.println("OK");
      } else if(bufferindex == 15 && serialbuffer[0] == 'S' && serialbuffer[1] == 'E' && serialbuffer[2] == 'T' && serialbuffer[3] == 'L' && serialbuffer[4] == 'I' && serialbuffer[5] == 'M' && 
                serialbuffer[6] == 'I' && serialbuffer[7] == 'T' && serialbuffer[8] == 'M' && serialbuffer[9] == 'A' && serialbuffer[10] == 'X' && serialbuffer[11] == 'C' &&
                serialbuffer[12] == ' ' && serialbuffer[13] >= '0' && serialbuffer[13] <= '9' && serialbuffer[14] >= '0' && serialbuffer[14] <= '9'){
        temperature_max2 = ((serialbuffer[13]-48)*10) + (serialbuffer[14]-48);
        eeprom_write_byte((uint8_t*)21, temperature_max2);
        Serial.println("OK");
      } else if(bufferindex == 14 && serialbuffer[0] == 'S' && serialbuffer[1] == 'E' && serialbuffer[2] == 'T' && serialbuffer[3] == 'L' && serialbuffer[4] == 'I' && serialbuffer[5] == 'M' && 
                serialbuffer[6] == 'I' && serialbuffer[7] == 'T' && serialbuffer[8] == 'M' && serialbuffer[9] == 'I' && serialbuffer[10] == 'N' && serialbuffer[11] == 'C' &&
                serialbuffer[12] == ' ' && serialbuffer[13] >= '0' && serialbuffer[13] <= '9'){
        temperature_min2 = serialbuffer[13]-48;
        eeprom_write_byte((uint8_t*)21, temperature_min2);
        Serial.println("OK");
      } else if(bufferindex == 15 && serialbuffer[0] == 'S' && serialbuffer[1] == 'E' && serialbuffer[2] == 'T' && serialbuffer[3] == 'L' && serialbuffer[4] == 'I' && serialbuffer[5] == 'M' && 
                serialbuffer[6] == 'I' && serialbuffer[7] == 'T' && serialbuffer[8] == 'M' && serialbuffer[9] == 'I' && serialbuffer[10] == 'N' && serialbuffer[11] == 'C' &&
                serialbuffer[12] == ' ' && serialbuffer[13] >= '0' && serialbuffer[13] <= '9' && serialbuffer[14] >= '0' && serialbuffer[14] <= '9'){
        temperature_min2 = ((serialbuffer[13]-48)*10) + (serialbuffer[14]-48);
        eeprom_write_byte((uint8_t*)22, temperature_min2);
        Serial.println("OK");
      } else {
        // Error, nastav timer1 na 1hz a pocitej 10 sekund
        open_delay = 0;
        TCNT1 = 0;
        OCR1A = 15624; // 1Hz
        TCCR1B = (1<<WGM12) | (1<<CS12) | (1<<CS10);
        bufferindex = 0;
        while(open_delay < 10){
          // Roluj text
          display(0, errorlookup[bufferindex]);
          display(1, errorlookup[bufferindex+1]);
          display(2, errorlookup[bufferindex+2]);
          display(3, errorlookup[bufferindex+3]);
          bufferindex++;
          if(bufferindex > 5)
            bufferindex = 0;
          _delay_ms(500); // prodleva pro rolovani
        }
        // vycisti display
        display(0, lookup[17]);
        display(1, lookup[17]);
        display(2, lookup[17]);
        display(3, lookup[17]);
      }
      while(Serial.available()){Serial.read();} // Vycisti buffer
      bufferindex = 0; // zresetuje buffer
    }
    timeout = 0;
  } else if (bufferindex > 0){
    timeout++;
  }

  // Zkontroluje otevreni lednice
  if(analogRead(A1) > LIGHT_THRESHOLD){
    // Lednice otevrena
    if(fridge_open) {
      // Zkontroluj timer1
      if(open_delay >= OPEN_DELAY){
        // spust poplach
        OCR1A = 1561; // 10Hz, frekvence se deli dvema
      }
    } else {
      // Lednice byla uzavrena - spust timer1 s 1hz
      TCCR1B |= (1<<CS12) | (1<<CS10);
      fridge_open = 1;
    }
  } else if (fridge_open){
    // Lednice uzavrena, ale byla otevrena - vynuluj timer1
    fridge_open = 0;
    open_delay = 0;
    PORTD &= ~(1<<6);
    TCCR1B = (1<<WGM12);
    OCR1A = 15624; // 1Hz
    TCNT1 = 0;
    // Vycisti display
    if(!alarm){
      display(0, lookup[17]);
      display(1, lookup[17]);
      display(2, lookup[17]);
      display(3, lookup[17]);
    }
  }

  // Zmena jednotek pres tlacitko menu a ulozeni do EEPROM
  if((PINB&switch_threshold&8)){
    switch_threshold &= ~8; // switch off
  } else if((~PINB)&(~switch_threshold)&8){
    switch_threshold |= 8; // switch on
    if(temperature_unit){
      temperature_unit = 0; // C
    } else {
      temperature_unit = 1; // F
    }
    eeprom_write_byte((uint8_t*)0, temperature_unit);
  }

  // Zobrazeni maximalni a minimalni teploty
  if((PINB&switch_threshold&4)){
    switch_threshold &= ~4; // switch off
  } else if((~PINB)&(~switch_threshold)&4){
    switch_threshold |= 4; // switch on
    displayTemperature(temperature_max); // Zobrazi maximalni teplotu
    _delay_ms(10000);
  }
  if((PINB&switch_threshold&2)){
    switch_threshold &= ~2; // switch off
  } else if((~PINB)&(~switch_threshold)&2){
    switch_threshold |= 2; // switch on
    displayTemperature(temperature_min); // Zobrazi maximalni teplotu
    _delay_ms(10000);
  }

  // Ziskani teploty
  uint16_t adc = analogRead(A0); // Ziska adc
  float resistance  = (float)(1023-adc)*10000/adc; // Spocte odpor
  int B = 3975; // Konstanta
  temperature = (int8_t)(1/(log(resistance/10000)/B+1/298.15)-273.15); // Spocte teplotu v C

  // Maximalni minimalni teploty
  if(temperature > temperature_max){
    temperature_max = temperature;
    eeprom_write_byte((uint8_t*)1, temperature_max);
    SaveTime(3); // Save time starting index 3
  }
  if(temperature < temperature_min){
    temperature_min = temperature;
    eeprom_write_byte((uint8_t*)2, temperature_min);
    SaveTime(10); // Save time starting index 10
  }

  // Zkontroluje mezni hodnoty
  if(temperature > temperature_max2 || temperature < temperature_min2){  // spusti alarm
    if(!alarm){
      alarm = 1;
      TCNT2 = 0;
      TCCR2B = 7; // prescaller 1024
      displayTemperature(temperature);
    }
  } else if (alarm){  // vypne alarm
    alarm = 0;
    TCCR2B = 0;
    PORTD &= ~60; // vypne led
    // Vycisti display
    display(0, lookup[17]);
    display(1, lookup[17]);
    display(2, lookup[17]);
    display(3, lookup[17]);
    displaytimeout = 0;
    ledtimeout = 0;
  } else {
    // Zobrazeni teploty, pokud lednice je otevrena, ve spravnych jednotkach
    if(fridge_open){
      displayTemperature(temperature);
    }
  }
}

// Pomocna funkce na prevod z C do F
int8_t convertToF(int8_t tempC){
  return ((9*(tempC))/5) + 32;
}

// Posle na seriovej port cas
void printTime(){
  Serial.print("20"); Serial.print(year); Serial.print("-");
  if(month < 10) {Serial.print("0");} Serial.print(month); Serial.print("-");
  if(day < 10) {Serial.print("0");} Serial.print(day); Serial.print(" ");
  if(hour < 10) {Serial.print("0");} Serial.print(hour); Serial.print(":");
  if(minute < 10) {Serial.print("0");} Serial.print(minute); Serial.print(":");
  if(sec < 10) {Serial.print("0");} Serial.print(sec); Serial.print(" ");
}

// Funkce k precteni casu z eeprom
void GetTimeEEPROM(uint8_t index){
  sec = eeprom_read_byte((uint8_t*)index); index++;
  minute = eeprom_read_byte((uint8_t*)index); index++;
  hour = eeprom_read_byte((uint8_t*)index); index++;
  day = eeprom_read_byte((uint8_t*)index); index++;
  month = eeprom_read_byte((uint8_t*)index); index++;
  year = eeprom_read_byte((uint8_t*)index); index++;
}

// Funkce k ulozeni casu do eeprom
void SaveTime(uint8_t index){
  getTime();
  eeprom_write_byte((uint8_t*)index, sec); index++;
  eeprom_write_byte((uint8_t*)index, minute); index++;
  eeprom_write_byte((uint8_t*)index, hour); index++;
  eeprom_write_byte((uint8_t*)index, day); index++;
  eeprom_write_byte((uint8_t*)index, month); index++;
  eeprom_write_byte((uint8_t*)index, year); index++;
}

// Pomocne funkce k ziskani casu
byte bcdToDec(byte val){
return ( (val/16*10) + (val%16) );
}
void getTime(){
Wire.beginTransmission(0x68);
Wire.write((byte)0x00);
Wire.endTransmission();  
Wire.requestFrom(0x68, 7);
sec = bcdToDec(Wire.read() & 0x7f);
minute = bcdToDec(Wire.read());
hour = bcdToDec(Wire.read() & 0x3f);
bcdToDec(Wire.read());
day = bcdToDec(Wire.read());
month = bcdToDec(Wire.read());
year = bcdToDec(Wire.read());
} 

// Pomocne funkce k zobrazeni cisla na displeji s TM1636
// Funkce jsou prevzate z dodane knihovny
void writeByte(int8_t wr_data){
  uint8_t i,count1;   
  for(i=0;i<8;i++)  {
    digitalWrite(7,LOW);      
    if(wr_data & 0x01)digitalWrite(8,HIGH);//LSB first
    else digitalWrite(8,LOW);
    wr_data >>= 1;      
    digitalWrite(7,HIGH);
  }  
  digitalWrite(7,LOW); //wait for the ACK
  digitalWrite(8,HIGH);
  digitalWrite(7,HIGH);     
  pinMode(8,INPUT);
  while(digitalRead(8)) { 
    count1 +=1;
    if(count1 == 200) {
     pinMode(8,OUTPUT);
     digitalWrite(8,LOW);
     count1 =0;
    }
    pinMode(8,INPUT);
  }
  pinMode(8,OUTPUT);
}
void start(void){
  digitalWrite(7,HIGH);//send start signal to TM1637
  digitalWrite(8,HIGH); 
  digitalWrite(8,LOW); 
  digitalWrite(7,LOW); 
} 
void stop(void){
  digitalWrite(7,LOW);
  digitalWrite(8,LOW);
  digitalWrite(7,HIGH);
  digitalWrite(8,HIGH); 
}
void display(uint8_t BitAddr,int8_t DispData){
  start();
  writeByte(0x44);
  stop();
  start();
  writeByte(BitAddr|0xc0);
  writeByte(DispData);
  stop();
  start();
  writeByte(0x88 + 7);
  stop();
}
void displayTemperature(int8_t temperature){
  if(temperature_unit)
    temperature = convertToF(temperature);
  if(temperature < 0){
    temperature = -temperature;
    display(0, lookup[16]);
  } else {
    if(temperature/100){
      display(0, lookup[temperature/100]);
    } else {
      display(0, lookup[17]);
    }
  }
  if((temperature%100)/10){
    display(1, lookup[(temperature%100)/10]);
  } else {
    display(1, lookup[17]);
  }
  display(2, lookup[temperature%10]);
  if(temperature_unit){
    display(3, lookup[15]); // F
  } else {
    display(3, lookup[12]); // C
  }
}

